;created 08/08/91
;updated 12/19/92

;======================================================================
;   Copyright (C) Copr. 1991, 1992 by Sidney J. Kelly
;           All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;=======================================================================
;RM92TS8,16,24,32,40,48

DOSSEG
.MODEL MEDIUM, BASIC

EXTRN B$STDL:FAR          ; QBASIC's STRINGRELEASE program

.CODE
; Please do not remove
;Copyright       DB    13,10,'Copyright Copr. (C) 1991 Sidney J. Kelly',13,10
;Copyright1      DB    'All Rights Reserved',13,10,26

;contains UPPER, LOWER, NULEN, NUASC
;contains KILLSTRING

;===========================================================================
; DECLARE SUB UPPER (Text$)
; Purpose:
;     Operates on source string and operates much faster than UCASE$
; approx 13% faster for a typical string on a 286 10 Mhz machine
; Counts from end of string to front because that is faster
;===========================================================================

EVEN
UPPER PROC FAR BASIC USES SI, STRING:WORD
	Mov     SI,STRING       ; get string descriptor in SI
	Mov     BX,[SI]         ; get length into BX
	Or      BX,BX           ; see if string has a 0 length
	Jz      Quit            ; if yes, then quit
	Mov     SI,[SI+2]       ; get address of string in SI
	; preload registers for faster comparison
	Mov     DX,'za'
	; must use BX because [SI+CX] as an index is not supported by
	; chip, while [SI+BX] is supported
EVEN                            ; speed loop on 286 machine
Redo:
	Dec     BX              ; reduce counter by 1
	Js      Quit            ; if hit less than zero then quit
	Mov     AL,[SI+BX]      ; load character into AL
	Cmp     AL,DL           ; is it < 'a'
	Jb      Redo            ; yes, so move to next letter
	Cmp     AL,DH           ; is it > 'z'
	Ja      Redo            ; yes, so move to next letter
	AND     AL,0DFh         ; else make character upper case
	Mov     [SI+BX],AL      ; store revised value
	Jmp     Short Redo      ; reloop
Quit:
	Ret
UPPER ENDP

;===========================================================================
; DECLARE SUB LOWER (Text$)
; Purpose:
;     Operates on source string and operates much faster than LCASE$
; approx 13% faster for a typical string on a 286 10 Mhz machine
; Counts from end of string to front because that is faster
;===========================================================================

EVEN
LOWER PROC FAR BASIC USES SI, STRING:WORD
	Mov     SI,STRING       ; get string descriptor in SI
	Mov     BX,[SI]         ; get length into BX
	Or      BX,BX           ; see if string has a 0 length
	Jz      Quit            ; if yes, then quit
	Mov     SI,[SI+2]       ; get address of string in SI
	; preload registers for faster comparison
	Mov     DX,'ZA'
EVEN                            ; speed loop on 286 machine
Redo:
	Dec     BX              ; reduce counter by 1
	Js      Quit            ; if hit less than zero then quit
	Mov     AL,[SI+BX]      ; load character into AL
	Cmp     AL,DL           ; is it < 'A'
	Jb      Redo            ; yes, so move to next letter
	Cmp     AL,DH           ; is it > 'Z'
	Ja      Redo            ; yes, so move to next letter
	OR      AL,20h          ; make character lower case
	Mov     [SI+BX],AL      ; store revised value
	Jmp     Short Redo      ; reloop
Quit:
	Ret
LOWER ENDP

;===========================================================================
; DECLARE FUNCTION NULEN% (Text$)
; Purpose:
;     Operates on source string and operates faster than LEN
; approx 62% faster for a typical string on a 286 10 Mhz machine
; Only works with strings, not TYPEs
;===========================================================================

EVEN
NULEN PROC FAR
	Mov     BX,SP           ; get stack pointer
	Mov	BX,[BX+4]	; get address of string descriptor
	Mov     AX,[BX]         ; get length into AX
	Ret	2		; pop string pointer off of stack
NULEN ENDP

;============================================================================
; DECLARE FUNCTION NUASC% (String$)
;       It won't crash if you feed it a null
;       string.  If LEN(String$) = 0 then returns 0.
;       Else it returns the ASC value.
;============================================================================

EVEN
NUASC PROC FAR
	Mov	BX,SP		; get stack pointer
	Xor	AX,AX           ; assume zero length string
	Mov     BX,[BX+4]	; get address of string descriptor
	Mov     CX,[BX]         ; move len of String to CX
	Jcxz    Quit            ; if LEN = 0 then quit (18 clocks on 8088)
	Mov     BX,[BX+2]       ; get address to BX
	Mov     AL,[BX]         ; get value of 1st char to AL
				; (AH already 0)
Quit:
	Ret	2		; pop string pointer off of stack
NUASC ENDP

;============================================================================
; DECLARE SUB KILLSTRING (Text$)
; Fast routine that makes input string a null string
;============================================================================

EVEN
KILLSTRING PROC FAR
	Mov	BX,SP		; get stack pointer address
	Push	[BX+4]		; push address of Text$
	CALL    B$STDL          ; call BASIC to delete string
	Ret	2		; remove pointer from stack
KILLSTRING ENDP
END

